<?php
/**
 * apc.class.php
 *
 * AurorisNET Software
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details. 
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301  USA
 */

require_once dirname(__FILE__)."/../pfccontainerinterface.class.php";

/**
 * pfcContainer_Apc is a container which stows data into an APC cache (if available)
 */
class pfcContainer_Apc extends pfcContainerInterface
{
    var $_users = array("nickid"    => array(),
                      "timestamp" => array());
    var $_meta = array();

    function pfcContainer_Apc()
    {
        pfcContainerInterface::pfcContainerInterface();
    }
  
    function getDefaultConfig()
    {
        // No configuration
        $cfg = pfcContainerInterface::getDefaultConfig();
        return $cfg;
    }
  
    function init(&$c)
    {
        $errors = pfcContainerInterface::init($c);

        if (ini_get("apc.enabled") != 1) 
        {
            $errors[] = _pfc("APC is not available.");
        }

        return $errors;
    }

    function setMeta($group, $subgroup, $leaf, $leafvalue = NULL)
    {
        // create or replace metadata file
        $leafname = "phpfreechat/" . $group . '/' .$subgroup . "/" . $leaf;
        $leafexists = $this->_apcexists($leafname);

        if ($leafvalue == NULL && $leafexists)
        {
            apc_store($leafname, "");
        }
        else
        {
            apc_store($leafname, $leafvalue);
        }

        if ($leafexists)
        {
            return 1; // value overwritten
        }
        else
        {
            return 0; // value created
        }
    }
  
    function getMeta($group, $subgroup = null, $leaf = null, $withleafvalue = false)
    {
        $ret = array();
        $ret["timestamp"] = array();
        $ret["value"]     = array();

        $leafgroup = "phpfreechat/" . $group;
        $leafsubgroup = $leafgroup . "/" . $subgroup;
        $leafname = $leafsubgroup . "/" . $leaf;

        if ($subgroup == NULL)
        {
            return $this->_apcmtime_group($leafgroup, 2);
        }
        
        if ($leaf == NULL)
        {
            return $this->_apcmtime_group($leafsubgroup, 3);
        }

        if (!$this->_apcexists($leafname))
        {
            return $ret;
        }

        if ($withleafvalue)
        {
            $ret["value"][] = apc_fetch($leafname);
        }
        else
        {
            $ret["value"][] = NULL;
        }

        $ret["timestamp"][] = $this->_apcmtime($leafname);

        return $ret;
    }

    function rmMeta($group, $subgroup = null, $leaf = null)
    {
        $leafall = "phpfreechat";
        $leafgroup = "phpfreechat/" . $group;
        $leafsubgroup = $leafgroup . "/" . $subgroup;
        $leafname = $leafsubgroup . "/" . $leaf;

        if ($group == NULL)
        {
            $this->_apcdelete($leafall);
        }

        if ($subgroup == NULL)
        {
            $this->_apcdelete($leafgroup);
        }

        if ($leaf == NULL)
        {
            $this->_apcdelete($leafsubgroup);
        }

        if (!$this->_apcexists($leafname)) 
        {
            return false;
        }

        apc_delete($leafname);

        return true;
    }

    // Checks the APC cache for an entry
    function _apcexists($entry)
    {
        $apcinfo = apc_cache_info("user");
        for ($i = 0; $i < count($apcinfo["cache_list"]); $i ++)
        {
            if ($apcinfo["cache_list"][$i]["info"] == $entry)
            {
                return true;
            }
        }
        
        return false;
    }
    
    // Finds the last modified time for an entry
    function _apcmtime($entry)
    {
        $apcinfo = apc_cache_info("user");
        for ($i = 0; $i < count($apcinfo["cache_list"]); $i ++)
        {
            if ($apcinfo["cache_list"][$i]["info"] == $entry)
                return $apcinfo["cache_list"][$i]["mtime"];
        }
        
        return 0;
    }

    // Finds the last modified time for a group of entries (partial name match)
    function _apcmtime_group($entry, $level)
    {
        $ret = array();
        $ret["timestamp"] = array();
        $ret["value"]     = array();

        $apcinfo = apc_cache_info("user");
        
        for ($i = 0; $i < count($apcinfo["cache_list"]); $i ++)
        {
            if(stristr($apcinfo["cache_list"][$i]["info"], $entry . "/") == true)
            {
                $ret["timestamp"][] = $apcinfo["cache_list"][$i]["mtime"];
                $path = explode("/", $apcinfo["cache_list"][$i]["info"]);
                $ret["value"][] = $path[$level];
            }
        }
        
        array_multisort($ret["value"], SORT_ASC, SORT_STRING, $ret["timestamp"], SORT_ASC, SORT_NUMERIC);

        $array_count = count($ret);
        $array_count_inner = count($ret["value"]);
        
        for ($i=0; $i<$array_count_inner; $i++) 
        {
            for ($j=$i+1; $j<$array_count_inner; $j++) 
            {
                if ($ret["value"][$i]==$ret["value"][$j]) 
                {
                    unset($ret["value"][$i]);
                    unset($ret["timestamp"][$i]);
                }
            }
        }

        return $ret;
    }
    
    // Deletes entries from the cache (all, group, subgroup only)
    function _apcdelete($entry)
    {
        $apcinfo = apc_cache_info("user");
        for ($i = 0; $i < count($apcinfo["cache_list"]); $i ++)
        {
            if(stristr($apcinfo["cache_list"][$i]["info"], $entry . "/") == true)
            {
                apc_delete($apcinfo["cache_list"][$i]["info"]);
            }
        }
    }
}

?> 